@@ -93,7 +93,7 @@ About.prototype.beginEvent = function() {
     this.initAboutLicenses(textBrowser);
 
     dialog.exec();
-    destr(dialog);
+    destrDialog(dialog);
     EAction.activateMainWindow();
 };
 
@@ -158,8 +158,8 @@ About.prototype.initAboutApp = function(textBrowser) {
             + "<p/>"
             + "<p>" + qsTr("All brand or product names are trademarks or registered trademarks of their respective holders.") + "</p>"
             + "<p/>"
-            + "<p>© 2011-2022 RibbonSoft, GmbH.</p>"
-            + "<p>" + qsTr("Portions of this software © %1 The Qt Company Ltd.").arg("2008-2022") + "</p>"
+            + "<p>© 2011-2024 RibbonSoft, GmbH.</p>"
+            + "<p>" + qsTr("Portions of this software © %1 The Qt Company Ltd.").arg("2008-2024") + "</p>"
             + "<p>" + qsTr("Qt ships with QCAD under the <a href='%1'>LGPL Open Source license</a> which confers various rights to you as the user, including the right to recompile the Qt libraries for your platform. To do that follow the <a href='%2'>documentation shown on the Qt website</a>.").arg("https://www.gnu.org/licenses/lgpl-3.0.html").arg("https://doc.qt.io/qt-5/build-sources.html") + "</p>"
             + "<p>" + qsTr("The program is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.") + "</p>"
             + "<p/>"
@@ -495,7 +495,7 @@ About.prototype.initAboutSystem = function(textEdit) {
     text += "\nNegative sign: " + sysloc.negativeSign();
     text += "\nPositive sign: " + sysloc.positiveSign();
     text += "\nText direction: " + sysloc.textDirection();
-    if (RSettings.getQtVersion() > 0x060000) {
+    if (RSettings.getQtVersion() >= 0x060000) {
         // TODO #qt6: prints number not name
         text += "\nSystem codec: " + QStringConverter.nameForEncoding(QStringConverter.System);
     }
@@ -546,6 +546,29 @@ About.prototype.initAboutLicenses = function(textBrowser) {
             + "<h1>%1</h1>".arg(qsTr("Licenses"))
             + "<hr/>";
 
+    if (RSettings.getQtVersion() >= 0x060000) {
+        html += "<h2>qtjsapi</h2>"
+        html += "<p>Copyright 2024 RibbonSoft, GmbH</p>"
+              + "<p>The MIT License (MIT)</p>"
+              + "<p>Permission is hereby granted, free of charge, to any person obtaining a "
+              + "copy of this software and associated documentation files (the “Software”), "
+              + "to deal in the Software without restriction, including without limitation "
+              + "the rights to use, copy, modify, merge, publish, distribute, sublicense, "
+              + "and/or sell copies of the Software, and to permit persons to whom the "
+              + "Software is furnished to do so, subject to the following conditions:"
+              + "<br/><br/>"
+              + "The above copyright notice and this permission notice shall be included in "
+              + "all copies or substantial portions of the Software."
+              + "<br/><br/>"
+              + "THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR "
+              + "IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, "
+              + "FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE "
+              + "AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER "
+              + "LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING "
+              + "FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER "
+              + "DEALINGS IN THE SOFTWARE.</p>";
+    }
+
     html += "<h2>libspatialindex</h2>"
     html += "<p>Permission is hereby granted, free of charge, to any person obtaining a "
           + "copy of this software and associated documentation files (the \"Software\"), "
