@@ -76,6 +76,13 @@ function requiresFramework(minVersion, maxVersion) {
     }
 }
 
+function checkMinVersion(minVersion) {
+    var frameworkV = getVersionNumber(RSettings.getMajorVersion(), RSettings.getMinorVersion(), RSettings.getRevisionVersion(), RSettings.getBuildVersion());
+    var minV = getVersionNumber(minVersion[0], minVersion[1], minVersion[2], minVersion[3]);
+
+    return frameworkV >= minV;
+}
+
 function getVersionNumber(major, minor, revision, build) {
     return major*1e9 + minor*1e6 + revision*1e3 + build;
 }
@@ -129,14 +136,29 @@ function isOfType(obj, type) {
  * Destroys the given object with C++ ownership.
  */
 function destr(obj) {
-    if (!RSettings.isQt(6)) {
-        obj.destroy();
-    }
-    else {
+    if (RSettings.getQtVersion() >= 0x060000) {
         // destroy does not work in Qt 6 / QJSEngine:
         // ("Invalid attempt to destroy() an indestructible object")
         obj.destr();
     }
+    else {
+        if (isFunction(obj.destroy)) {
+            obj.destroy();
+        }
+    }
+}
+
+function destrDialog(dlg) {
+    if (RSettings.getQtVersion() >= 0x060000) {
+        // delete later in Qt 6
+        // immediate delete might cause unpredictable crashes in event processing:
+        dlg.deleteLater();
+    }
+    else {
+        if (isFunction(dlg.destroy)) {
+            dlg.destroy();
+        }
+    }
 }
 
 /**
@@ -150,13 +172,16 @@ function getPtr(p) {
         if (isFunction(p.data)) {
             return p.data();
         }
+        else {
+            return p;
+        }
     }
 }
 
 /**
  * Checks if the given object is undefined or null.
  *
- * \return true if the given object is defined and not null.
+ * \return false if the given object is defined and not null.
  * This method also supports shared pointers, i.e. obj.isNull()==true.
  */
 function isNull(obj) {
@@ -272,6 +297,15 @@ function isQWidget(obj) {
     return (isQObject(obj) && typeof(obj.show)==="function");
 }
 
+/**
+ * Checks if the given object is a valid QAction.
+ *
+ * \return true if the given object is a QAction.
+ */
+function isQAction(obj) {
+    return (isQObject(obj) && typeof(obj.actionGroup)==="function");
+}
+
 /**
  * \return true if the given object is a separator (a frame of type vertical line).
  */
@@ -568,7 +602,12 @@ function isEllipseEntity(obj) {
  * \return true if the given object is an ellipse arc entity (REllipseEntity).
  */
 function isEllipseArcEntity(obj) {
-    return (isOfType(obj, REllipseEntity) || isOfType(obj, REllipseEntityPointer)) && !obj.isFullEllipse();
+    if (RSettings.getQtVersion() < 0x060000) {
+        return (isOfType(obj, REllipseEntity) || isOfType(obj, REllipseEntityPointer)) && !obj.isFullEllipse();
+    }
+    else {
+        return isOfType(obj, REllipseEntity);
+    }
 }
 
 /**
@@ -671,6 +710,18 @@ function isPolylineEntity(obj) {
     }
 }
 
+/**
+ * \return true if the given object is a wipeout entity (RWipeoutEntity).
+ */
+function isWipeoutEntity(obj) {
+    if (RSettings.getQtVersion() < 0x060000) {
+        return isOfType(obj, RWipeoutEntity) || isOfType(obj, RWipeoutEntityPointer);
+    }
+    else {
+        return isOfType(obj, RWipeoutEntity);
+    }
+}
+
 /**
  * \return true if the given object is a geometrically closed polyline entity.
  */
@@ -1216,6 +1267,10 @@ function entityTypeToClass(type) {
         return "RTextEntity";
     case RS.EntityViewport:
         return "RViewportEntity";
+    case RS.EntityWipeout:
+        return "RWipeoutEntity";
+    case RS.EntityXRef:
+        return "RXRefEntity";
     default:
         return undefined;
     }
@@ -1276,9 +1331,9 @@ function entityTypeToString(type, plural) {
     case RS.EntityDimAngular:
         return plural ? qsTr("Angular Dimensions") : qsTr("Angular Dimension");
     case RS.EntityDimAngular2L:
-        return plural ? qsTr("Angular Dimensions (2 Line)") : qsTr("Angular Dimension (2 Line)");
+        return plural ? qsTr("Angular Dimensions (2 Lines)") : qsTr("Angular Dimension (2 Lines)");
     case RS.EntityDimAngular3P:
-        return plural ? qsTr("Angular Dimensions (3 Point)") : qsTr("Angular Dimension (3 Point)");
+        return plural ? qsTr("Angular Dimensions (3 Points)") : qsTr("Angular Dimension (3 Points)");
     case RS.EntityDimArcLength:
         return plural ? qsTr("Arc Dimension") : qsTr("Arc Dimension");
     case RS.EntityDimDiametric:
@@ -1323,6 +1378,10 @@ function entityTypeToString(type, plural) {
         return plural ? qsTr("Texts") : qsTr("Text");
     case RS.EntityViewport:
         return plural ? qsTr("Viewports") : qsTr("Viewport");
+    case RS.EntityWipeout:
+        return plural ? qsTr("Wipeouts") : qsTr("Wipeout");
+    case RS.EntityXRef:
+        return plural ? qsTr("External References") : qsTr("External Reference");
     case RS.EntityUnknown:
     default:
         return plural ? qsTr("Unknown Entities") : qsTr("Unknown Entity");
@@ -1420,6 +1479,10 @@ function getEntityType(typeName) {
         return RS.EntityText;
     case "Viewport":
         return RS.EntityViewport;
+    case "Wipeout":
+        return RS.EntityWipeout;
+    case "External Reference":
+        return RS.EntityXRef;
     default:
         return RS.ObjectUnknown;
     }
@@ -1481,7 +1544,19 @@ function objectFromPath(path) {
  */
 function getWidgetPath(widget) {
     var pw = widget.parentWidget();
-    var str = widget.objectName !== "" ? widget.objectName : widget.toString();
+    var str = "";
+    if (RSettings.getQtVersion()>=0x060000) {
+        if (widget.objectName !== "") {
+            str = widget.objectName;
+        }
+        else {
+            return getWidgetPath(pw);
+        }
+    }
+    else {
+        str = widget.objectName !== "" ? widget.objectName : widget.toString();
+    }
+
     if (isNull(pw)) {
         return str;
     }
@@ -1521,7 +1596,7 @@ function getWidgets(widget, ca, allowDuplicates) {
     var children = widget.children();
     for (var i = 0; i < children.length; ++i) {
         var child = children[i];
-        if (isDeleted(child)) {
+        if (isNull(child) || isDeleted(child)) {
             continue;
         }
         if (!isNull(child.objectName) && child.objectName !== "") {
@@ -1865,6 +1940,12 @@ Array.prototype.remove = function(val, compareFunction) {
     }
 };
 
+Array.prototype.removeAt = function(index) {
+    var part1 = this.slice(0, index);
+    var part2 = this.slice(index + 1);
+    return (part1.concat(part2));
+};
+
 /**
  * Removes all matches of the given value from the array.
  * \param The value(s) to remove.
@@ -1923,7 +2004,7 @@ Array.prototype.clone = function() {
  */
 Array.prototype.sortNumerical = function() {
     Array.prototype.sort.call(this, function(a,b) { return a - b });
-}
+};
 
 Array.alphaNumericalSorter = function(a, b) {
     return RS.compareAlphanumerical(a, b);
@@ -1965,6 +2046,32 @@ Array.prototype.filter = function(func, thisArg) {
     return res;
 };
 
+Array.prototype.flat = function(depth) {
+    // If no depth is specified, default to 1
+    if (depth === undefined) {
+        depth = 1;
+    }
+
+    // Recursively reduce sub-arrays to the specified depth
+    var flatten = function (arr, depth) {
+
+        // If depth is 0, return the array as-is
+        if (depth < 1) {
+            return arr.slice();
+        }
+
+        // Otherwise, concatenate into the parent array
+        return arr.reduce(function (acc, val) {
+            return acc.concat(Array.isArray(val) ? flatten(val, depth - 1) : val);
+        }, []);
+
+    };
+
+    return flatten(this, depth);
+
+};
+
+
 /**
  * Sorts this array with string items (case insensitive)
  */
@@ -2262,6 +2369,54 @@ String.prototype.regexLastIndexOf = function(regex, startpos) {
     return lastIndexOf;
 };
 
+String.prototype.padStart = function(targetLength, padString) {
+    targetLength = targetLength >> 0; //floor if number or convert non-number to 0;
+    padString = String(typeof padString !== 'undefined' ? padString : ' ');
+    if (this.length > targetLength) {
+        return String(this);
+    } else {
+        targetLength = targetLength - this.length;
+        if (targetLength > padString.length) {
+            padString += padString.repeat(targetLength / padString.length); //append to original to ensure we are longer than needed
+        }
+        return padString.slice(0, targetLength) + String(this);
+    }
+};
+
+String.prototype.repeat = function(count) {
+    var str = '' + this;
+    // To convert string to integer.
+    count = +count;
+    // Check NaN
+    if (count != count)
+        count = 0;
+
+    if (count < 0)
+        throw new RangeError('repeat count must be non-negative');
+
+    if (count == Infinity)
+        throw new RangeError('repeat count must be less than infinity');
+
+    count = Math.floor(count);
+    if (str.length == 0 || count == 0)
+        return '';
+
+    // Ensuring count is a 31-bit integer allows us to heavily optimize the
+    // main part. But anyway, most current (August 2014) browsers can't handle
+    // strings 1 << 28 chars or longer, so:
+    if (str.length * count >= 1 << 28)
+        throw new RangeError('repeat count must not overflow maximum string size');
+
+    var maxCount = str.length * count;
+    count = Math.floor(Math.log(count) / Math.log(2));
+    while (count) {
+        str += str;
+        count--;
+    }
+    str += str.substring(0, maxCount - str.length);
+    return str;
+};
+
 /**
  * Modulo which works also for negative numbers (workaround for
  * ECMAScript modulo bug).
@@ -2414,43 +2569,40 @@ function stringToDirectDistanceEntry(relativeZero, cursorPosition, str) {
  * Creates and returns a new entity based on the given shape.
  */
 function shapeToEntity(document, shape) {
+    if (isNull(shape)) {
+        return undefined;
+    }
+
     var s = shape;
     if (isFunction(s.data)) {
         s = s.data();
     }
 
-    if (isPointShape(s)) {
+    switch (s.getShapeType()) {
+    case RShape.Point:
         return new RPointEntity(document, new RPointData(s.getPosition()));
-    }
-    else if (isLineShape(s)) {
+    case RShape.Line:
         return new RLineEntity(document, new RLineData(s));
-    }
-    else if (isRayShape(s)) {
+    case RShape.Ray:
         return new RRayEntity(document, new RRayData(s));
-    }
-    else if (isXLineShape(s)) {
+    case RShape.XLine:
         return new RXLineEntity(document, new RXLineData(s));
-    }
-    else if (isArcShape(s)) {
+    case RShape.Arc:
         return new RArcEntity(document, new RArcData(s));
-    }
-    else if (isCircleShape(s)) {
+    case RShape.Circle:
         return new RCircleEntity(document, new RCircleData(s));
-    }
-    else if (isEllipseShape(s)) {
+    case RShape.Ellipse:
         return new REllipseEntity(document, new REllipseData(s));
-    }
-    else if (isPolylineShape(s)) {
+    case RShape.Polyline:
         return new RPolylineEntity(document, new RPolylineData(s));
-    }
-    else if (isSplineShape(s)) {
+    case RShape.Spline:
         return new RSplineEntity(document, new RSplineData(s));
-    }
-    else if (isTriangleShape(s)) {
+    case RShape.Triangle:
         return new RSolidEntity(document, new RSolidData(s));
+    default:
+        return undefined;
     }
 
-    qWarning("shapeToEntity: unknown shape: ", s);
     return undefined;
 }
 
@@ -2577,7 +2729,12 @@ function getMainWindow() {
 }
 
 function getCurrentDateTime(format) {
-    return new QDateTime(QDateTime.currentDateTime()).toString(format);
+    if (RSettings.getQtVersion() >= 0x060000) {
+        return QDateTime.currentDateTime().toString(format);
+    }
+    else {
+        return new QDateTime(QDateTime.currentDateTime()).toString(format);
+    }
 }
 
 function mergeProperties(obj1,obj2) {
@@ -2777,7 +2934,7 @@ function addActionsToWidgets() {
                 if (isNull(scObjs[si])) {
                     continue;
                 }
-                scObjs[si].destroy();
+                destr(scObjs[si]);
             }
         }
         appWin.setProperty("DelegatedShortcutsObjs", []);
@@ -2829,42 +2986,6 @@ function addActionsToWidgets() {
             }
             var w = appWin.findChild(wn);
             if (!isNull(w)) {
-
-                // workaround for QTBUG-38256 (action not triggered for letter based shortcuts in SUB menus):
-                // workaround for QTBUG-57990 (action not triggered for letter based shortcuts in ALL menus):
-                /*
-                if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
-                    (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
-
-                    if (k===0) {
-                        var shortcuts = a.shortcuts();
-
-                        var scObjs = appWin.property("DelegatedShortcutsObjs");
-                        if (isNull(scObjs)) {
-                            scObjs=[];
-                        }
-                        for (var si=0; si<shortcuts.length; si++) {
-                            var scObj = new QShortcut(shortcuts[si], a.parentWidget(), 0, 0, Qt.WindowShortcut)
-                            scObj.activated.connect(a, "trigger");
-                            scObjs.push(scObj);
-                        }
-                        appWin.setProperty("DelegatedShortcutsObjs", scObjs);
-
-                        var sc = a.shortcut.toString();
-                        if (sc.length>0) {
-                            if (RS.getSystemId()==="linux") {
-                                a.setShortcutText("[" + sc + "]");
-                            }
-                            else {
-                                a.setShortcutText(" " + sc);
-                            }
-                        }
-                        a.setProperty("DelegatedShortcuts", shortcuts);
-                        a.setDefaultShortcuts([]);
-                    }
-                }
-                */
-
                 if (visibility) {
                     RGuiAction.addToWidget(a, w);
                     addedToWidget = true;
@@ -3065,10 +3186,10 @@ function setUtf8Codec(ts) {
 
 function readTextFile(fileName) {
     var file = new QFile(fileName);
-    var flags = new QIODevice.OpenMode(QIODevice.ReadOnly | QIODevice.Text);
+    var flags = makeQIODeviceOpenMode(QIODevice.ReadOnly, QIODevice.Text);
     if (file.open(flags)) {
         var textStream = new QTextStream(file);
-        textStream.setCodec("UTF-8");
+        setUtf8Codec(textStream);
         var contents = textStream.readAll();
         file.close();
         return contents;
@@ -3079,10 +3200,10 @@ function readTextFile(fileName) {
 
 function writeTextFile(fileName, str) {
     var file = new QFile(fileName);
-    var flags = new QIODevice.OpenMode(QIODevice.WriteOnly | QIODevice.Text);
+    var flags = makeQIODeviceOpenMode(QIODevice.WriteOnly, QIODevice.Text);
     if (file.open(flags)) {
         var textStream = new QTextStream(file);
-        textStream.setCodec("UTF-8");
+        setUtf8Codec(textStream);
         textStream.writeString(str);
         file.close();
     }
@@ -3202,6 +3323,42 @@ function makeQtItemFlags() {
     return makeFlags.apply(this, argumentsNew);
 }
 
+function makeQtKeyboardModifiers() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(Qt.KeyboardModifiers);
+    return makeFlags.apply(this, argumentsNew);
+}
+
+function makeQtMouseButtons() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(Qt.MouseButtons);
+    return makeFlags.apply(this, argumentsNew);
+}
+
+function makeQEventLoopProcessEventsFlags() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(QEventLoop.ProcessEventsFlags);
+    return makeFlags.apply(this, argumentsNew);
+}
+
+function makeQItemSelectionModelSelectionFlags() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(QItemSelectionModel.SelectionFlags);
+    return makeFlags.apply(this, argumentsNew);
+}
+
+function makeQAbstractItemViewEditTriggers() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(QAbstractItemView.EditTriggers);
+    return makeFlags.apply(this, argumentsNew);
+}
+
+function makeQDirIteratorIteratorFlags() {
+    var argumentsNew = [].slice.call(arguments, 0);
+    argumentsNew.unshift(QDirIterator.IteratorFlags);
+    return makeFlags.apply(this, argumentsNew);
+}
+
 function makeFlags() {
     if (RSettings.isQt(6)) {
         var ret = 0;
@@ -3217,13 +3374,154 @@ function makeFlags() {
     }
 }
 
-function getQLocaleName(locale) {
-    if (isFunction(locale.name)) {
-        return locale.name();
+function getRGraphicsView(view) {
+    if (RSettings.getQtVersion()>=0x060000) {
+        return view;
     }
     else {
-        // QJSEngine does not allow "name" as function.
-        return locale.name2();
+        if (isFunction(view.getRGraphicsView)) {
+            return view.getRGraphicsView();
+        }
+        else {
+            return view;
+        }
+    }
+}
+
+/**
+ * Open files given as arguments args
+ *
+ * \param createNew Creates a new document if no files are given
+ * \param close Closes existing open MDI widgets
+ */
+function openFiles(args, createNew, close) {
+    qDebug("openFiles: " + args);
+
+    var appWin = RMainWindowQt.getMainWindow();
+    if (isNull(appWin)) {
+        // application is shutting down..
+        return;
+    }
+    var mdiArea = appWin.getMdiArea();
+    var mdiChildren = mdiArea.subWindowList();
+    var foundFile = false;
+    var filter = undefined;
+
+    for (var i = 0; i < args.length; ++i) {
+        // arguments with one parameter:
+        if (args[i] === "-locale" || args[i] === "-autostart"
+            || args[i] === "-app-id" || args[i] === "-ignore"
+            || args[i] === "-config") {
+            // skip 2 arguments
+            i++;
+            if (i>=args.length) {
+                break;
+            }
+            continue;
+        }
+
+        // argument with two parameters
+        if (args[i] === "-font-substitution" || args[i] === "-fs" || args[i] === "-ts") {
+            // skip 3 arguments
+            i+=2;
+            if (i>=args.length) {
+                break;
+            }
+            continue;
+        }
+
+        if (isNull(args[i])) {
+            continue;
+        }
+
+        // all arguments after -exec are script files or script arguments:
+        if (args[i] === "-exec") {
+            break;
+        }
+
+        if (args[i] === "-filter") {
+            if (++i>=args.length) {
+                break;
+            }
+            filter = args[i];
+            continue;
+        }
+
+
+        // skip other arguments without parameter:
+        if (args[i][0] === "-") {
+            continue;
+        }
+
+        foundFile = true;
+        var foundExisting = false;
+
+        var arg = args[i];
+        var isLocalFile = true;
+
+        if (isUrl(arg)) {
+            var url = new QUrl(arg);
+            if (url.isLocalFile()) {
+                // arg is now a path:
+                arg = url.toLocalFile();
+            }
+            else {
+                isLocalFile = false;
+            }
+        }
+
+        if (isLocalFile) {
+            // if the file is already open, activate that appropriate sub window instead
+            // of opening the file again:
+            var document = undefined;
+            var fileName = undefined;
+            var fileInfo = undefined;
+            var argFileInfo = undefined;
+            for (var k=0; k<mdiChildren.length; k++) {
+                document = mdiChildren[k].getDocument();
+                fileName = document.getFileName();
+                fileInfo = new QFileInfo(fileName);
+                argFileInfo = new QFileInfo(getAbsolutePathForArg(arg));
+
+                if (fileInfo.absoluteFilePath()===argFileInfo.absoluteFilePath()) {
+                    mdiArea.setActiveSubWindow(mdiChildren[k]);
+                    if (close) {
+                        mdiArea.closeActiveSubWindow();
+                    }
+                    else {
+                        foundExisting = true;
+                    }
+                    break;
+                }
+            }
+        }
+
+        // open the file if it is not already open:
+        if (!foundExisting) {
+            if (isLocalFile) {
+                NewFile.createMdiChild(getAbsolutePathForArg(arg), filter);
+            }
+            else {
+                NewFile.createMdiChild(arg, filter);
+            }
+        }
+    }
+
+    // create new document if no files were loaded:
+    if (!foundFile && createNew===true) {
+        var fileNewAction = RGuiAction.getByScriptFile("scripts/File/NewFile/NewFile.js");
+        if (!isNull(fileNewAction)) {
+            fileNewAction.slotTrigger();
+        }
+    }
+}
+
+function createValidator(rx, parent) {
+    if (RSettings.getQtVersion()>=0x060000) {
+        return new QRegularExpressionValidator(new QRegularExpression(rx), parent);
+    }
+    else {
+        return new QRegExpValidator(new RegExp(rx), parent);
     }
 }
 
